@@ -14,7 +14,9 @@
 #include <wx/filename.h>
 
 #ifndef __WXMSW__
+	#define mutex mutex_override // Sadly on some platforms system headers include conflicting names
 	#include <netdb.h>
+	#undef mutex
 	#ifndef AI_IDN
 		#include <idna.h>
 		extern "C" {
@@ -1042,7 +1044,7 @@ int CRealControlSocket::ContinueConnect()
 
 	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
 	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy()) {
-		LogMessage(MessageType::Status, _("Connecting to %s through proxy"), m_pCurrentServer->FormatHost());
+		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), m_pCurrentServer->FormatHost(), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
 
 		host = engine_.GetOptions().GetOption(OPTION_PROXY_HOST);
 		port = engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT);
@@ -1226,8 +1228,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			if (engine_.GetDirectoryCache().LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dir_did_exist, matched_case) &&
 				matched_case)
 			{
-				wxLongLong size = entry.size;
-				pData->remoteFileSize = size.GetValue();
+				pData->remoteFileSize = entry.size;
 				if (entry.has_date())
 					pData->fileTime = entry.time;
 
