@@ -265,6 +265,57 @@ bool IsIpAddress(const wxString& address)
 	return true;
 }
 
+namespace {
+#if defined(__WXMSW__) && !defined(_MSC_VER)
+// Unfortunately MiNGW does not have a working random_device
+// Implement our own in terms of CryptGenRandom.
+// Fall back to time-seeded mersenne twister on error
+struct provider
+{
+	provider()
+	{
+		if (!CryptAcquireContextW(&h_, 0, 0, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT | CRYPT_SILENT)) {
+			h_ = 0;
+		}
+		mt_.seed(CDateTime::Now().GetTimeT());
+	}
+	~provider()
+	{
+		if (h_) {
+			CryptReleaseContext(h_, 0);
+		}
+	}
+
+	HCRYPTPROV h_{};
+	std::mt19937_64 mt_;
+};
+
+struct working_random_device
+{
+	typedef uint64_t result_type;
+
+	constexpr static result_type min() { return std::numeric_limits<result_type>::min(); }
+	constexpr static result_type max() { return std::numeric_limits<result_type>::max(); }
+
+	result_type operator()()
+	{
+		thread_local provider prov;
+
+		result_type ret{};
+		if (!prov.h_ || !CryptGenRandom(prov.h_, sizeof(ret), reinterpret_cast<BYTE*>(&ret))) {
+			ret = prov.mt_();
+		}
+
+		return ret;
+	}
+};
+
+static_assert(working_random_device::max() == std::mt19937_64::max(), "Unsupported std::mt19937_64::max()");
+static_assert(working_random_device::min() == std::mt19937_64::min(), "Unsupported std::mt19937_64::min()");
+#else
+typedef std::random_device working_random_device;
+#endif
+}
 
 int GetRandomNumber(int min, int max)
 {
@@ -272,10 +323,9 @@ int GetRandomNumber(int min, int max)
 	if (min >= max)
 		return min;
 
-	static std::mt19937_64 gen = std::mt19937_64(std::random_device()());
 	std::uniform_int_distribution<int> dist(min, max);
-
-	return dist(gen);
+	working_random_device rd;
+	return dist(rd);
 }
 
 void MakeLowerAscii(wxString& str)
