@@ -21,14 +21,13 @@ void log_func(int level, const char* msg)
 }
 #endif
 
-CTlsSocket::CTlsSocket(CEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
+CTlsSocket::CTlsSocket(CEventHandler* pEvtHandler, CSocket& socket, CControlSocket* pOwner)
 	: CEventHandler(pOwner->event_loop_)
 	, CBackend(pEvtHandler)
 	, m_pOwner(pOwner)
-	, m_pSocket(pSocket)
+	, m_socket(socket)
 {
-	wxASSERT(pSocket);
-	m_pSocketBackend = new CSocketBackend(this, *m_pSocket, m_pOwner->GetEngine().GetRateLimiter());
+	m_pSocketBackend = new CSocketBackend(this, m_socket, m_pOwner->GetEngine().GetRateLimiter());
 
 	m_implicitTrustedCert.data = 0;
 	m_implicitTrustedCert.size = 0;
@@ -99,7 +98,7 @@ bool CTlsSocket::InitSession()
 		return false;
 	}
 
-	gnutls_dh_set_prime_bits(m_session, 512);
+	gnutls_dh_set_prime_bits(m_session, 1024);
 
 	gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, m_certCredentials);
 
@@ -307,7 +306,6 @@ void CTlsSocket::operator()(CEventBase const& ev)
 
 void CTlsSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
-	wxASSERT(m_pSocket);
 	if (!m_session)
 		return;
 
@@ -327,8 +325,7 @@ void CTlsSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error
 			if (peeked >= 0) {
 				if (peeked > 0)
 					m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing close event"));
-				else
-				{
+				else {
 					m_socket_eof = true;
 					m_socketClosed = true;
 				}
@@ -368,8 +365,7 @@ void CTlsSocket::OnRead()
 		ContinueHandshake();
 	if (m_tlsState == TlsState::closing)
 		ContinueShutdown();
-	else if (m_tlsState == TlsState::conn)
-	{
+	else if (m_tlsState == TlsState::conn) {
 		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
@@ -392,8 +388,7 @@ void CTlsSocket::OnSend()
 		ContinueHandshake();
 	else if (m_tlsState == TlsState::closing)
 		ContinueShutdown();
-	else if (m_tlsState == TlsState::conn)
-	{
+	else if (m_tlsState == TlsState::conn) {
 		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
@@ -441,24 +436,22 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 		// Implicitly trust certificate of primary socket
 		unsigned int cert_list_size;
 		const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(pPrimarySocket->m_session, &cert_list_size);
-		if (cert_list && cert_list_size)
-		{
+		if (cert_list && cert_list_size) {
 			delete [] m_implicitTrustedCert.data;
 			m_implicitTrustedCert.data = new unsigned char[cert_list[0].size];
 			memcpy(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size);
 			m_implicitTrustedCert.size = cert_list[0].size;
 		}
 
-		if (try_resume)
-		{
+		if (try_resume) {
 			if (!CopySessionData(pPrimarySocket))
 				return FZ_REPLY_ERROR;
 		}
 
-		hostname = pPrimarySocket->m_pSocket->GetPeerHost();
+		hostname = pPrimarySocket->m_socket.GetPeerHost();
 	}
 	else {
-		hostname = m_pSocket->GetPeerHost();
+		hostname = m_socket.GetPeerHost();
 	}
 
 	if( !hostname.empty() && !IsIpAddress(hostname) ) {
@@ -862,8 +855,8 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 		return false;
 	}
 
-	wxDateTime expirationTime = gnutls_x509_crt_get_expiration_time(cert);
-	wxDateTime activationTime = gnutls_x509_crt_get_activation_time(cert);
+	CDateTime expirationTime(gnutls_x509_crt_get_expiration_time(cert), CDateTime::seconds);
+	CDateTime activationTime(gnutls_x509_crt_get_activation_time(cert), CDateTime::seconds);
 
 	// Get the serial number of the certificate
 	unsigned char buffer[40];
@@ -1004,6 +997,11 @@ std::vector<wxString> CTlsSocket::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
 	return ret;
 }
 
+bool CTlsSocket::CertificateIsBlacklisted(std::vector<CCertificate> const& certificates)
+{
+	return false;
+}
+
 int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != TlsState::handshake) {
@@ -1020,7 +1018,9 @@ int CTlsSocket::VerifyCertificate()
 	}
 
 	unsigned int status = 0;
-	if (gnutls_certificate_verify_peers2(m_session, &status) < 0) {
+	int const verifyResult = gnutls_certificate_verify_peers2(m_session, &status);
+	if (verifyResult < 0) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("gnutls_certificate_verify_peers2 returned %d with status %u"), verifyResult, status);
 		m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
 		Failure(0, true);
 		return FZ_REPLY_ERROR;
@@ -1062,6 +1062,8 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_OK;
 	}
 
+	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
+
 	std::vector<CCertificate> certificates;
 	for (unsigned int i = 0; i < cert_list_size; ++i) {
 		CCertificate cert;
@@ -1075,6 +1077,11 @@ int CTlsSocket::VerifyCertificate()
 		++cert_list;
 	}
 
+	if (CertificateIsBlacklisted(certificates)) {
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
 	CCertificateNotification *pNotification = new CCertificateNotification(
 		m_pOwner->GetCurrentServer()->GetHost(),
 		m_pOwner->GetCurrentServer()->GetPort(),
@@ -1086,8 +1093,6 @@ int CTlsSocket::VerifyCertificate()
 
 	m_pOwner->SendAsyncRequest(pNotification);
 
-	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
-
 	return FZ_REPLY_WOULDBLOCK;
 }
 
