@@ -19,7 +19,7 @@ public:
 		closed
 	};
 
-	CTlsSocket(CEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner);
+	CTlsSocket(CEventHandler* pEvtHandler, CSocket& pSocket, CControlSocket* pOwner);
 	virtual ~CTlsSocket();
 
 	bool Init();
@@ -58,6 +58,7 @@ protected:
 	void ContinueShutdown();
 
 	int VerifyCertificate();
+	bool CertificateIsBlacklisted(std::vector<CCertificate> const& certificates);
 
 	TlsState m_tlsState{TlsState::noconn};
 
@@ -102,7 +103,7 @@ protected:
 	bool m_socketClosed{};
 
 	CSocketBackend* m_pSocketBackend{};
-	CSocket* m_pSocket{};
+	CSocket& m_socket;
 
 	bool m_shutdown_requested{};
 
