@@ -188,10 +188,10 @@ public:
 	{
 		m_pMainFrame = pMainFrame;
 
-		CContextManager::Get()->RegisterHandler(this, STATECHANGE_REMOTE_IDLE, false, true);
-		CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, false, true);
+		CContextManager::Get()->RegisterHandler(this, STATECHANGE_REMOTE_IDLE, false);
+		CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, false);
 
-		CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false, false);
+		CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false);
 	}
 
 protected:
@@ -256,21 +256,14 @@ CMainFrame::CMainFrame()
 	: m_engineContext(*COptions::Get())
 	, m_comparisonToggleAcceleratorId(wxNewId())
 {
-#ifdef __WXMAC__
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->Cut(); }, 'X');
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->Copy(); }, 'C');
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->Paste(); }, 'V');
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->SelectAll(); }, 'A');
-#endif
-
 	m_pActivityLed[0] = m_pActivityLed[1] = 0;
 
 	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CMainFrame"));
 	wxRect screen_size = CWindowStateManager::GetScreenDimensions();
 
 	wxSize initial_size;
-	initial_size.x = wxMin(900, screen_size.GetWidth() - 10);
-	initial_size.y = wxMin(750, screen_size.GetHeight() - 50);
+	initial_size.x = wxMin(1200, screen_size.GetWidth() - 10);
+	initial_size.y = wxMin(950, screen_size.GetHeight() - 50);
 
 	Create(NULL, -1, _T("FileZilla"), wxDefaultPosition, initial_size);
 	SetSizeHints(250, 250);
@@ -297,11 +290,10 @@ CMainFrame::CMainFrame()
 
 	// It's important that the context control gets created before our own state handler
 	// so that contextchange events can be processed in the right order.
-	m_pContextControl = new CContextControl(this);
+	m_pContextControl = new CContextControl(*this);
 
 	m_pStatusBar = new CStatusBar(this);
-	if (m_pStatusBar)
-	{
+	if (m_pStatusBar) {
 		m_pActivityLed[0] = new CLed(m_pStatusBar, 0);
 		m_pActivityLed[1] = new CLed(m_pStatusBar, 1);
 
@@ -313,8 +305,7 @@ CMainFrame::CMainFrame()
 
 	m_closeEventTimer.SetOwner(this);
 
-	if (CFilterManager::HasActiveFilters(true))
-	{
+	if (CFilterManager::HasActiveFilters(true)) {
 		if (COptions::Get()->GetOptionVal(OPTION_FILTERTOGGLESTATE))
 			CFilterManager::ToggleFilters();
 	}
@@ -362,29 +353,23 @@ CMainFrame::CMainFrame()
 
 	m_pContextControl->CreateTab();
 
-	switch (message_log_position)
-	{
+	switch (message_log_position) {
 	case 1:
 		m_pTopSplitter->Initialize(m_pBottomSplitter);
-		if (COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG))
-		{
+		if (COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG)) {
 			if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE))
 				m_pQueueLogSplitter->SplitVertically(m_pQueuePane, m_pStatusView);
-			else
-			{
+			else {
 				m_pQueueLogSplitter->Initialize(m_pStatusView);
 				m_pQueuePane->Hide();
 			}
 		}
-		else
-		{
-			if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE))
-			{
+		else {
+			if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE)) {
 				m_pStatusView->Hide();
 				m_pQueueLogSplitter->Initialize(m_pQueuePane);
 			}
-			else
-			{
+			else {
 				m_pQueuePane->Hide();
 				m_pStatusView->Hide();
 				m_pQueueLogSplitter->Hide();
@@ -395,8 +380,7 @@ CMainFrame::CMainFrame()
 		m_pTopSplitter->Initialize(m_pBottomSplitter);
 		if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE))
 			m_pQueueLogSplitter->Initialize(m_pQueuePane);
-		else
-		{
+		else {
 			m_pQueueLogSplitter->Hide();
 			m_pQueuePane->Hide();
 		}
@@ -405,15 +389,13 @@ CMainFrame::CMainFrame()
 	default:
 		if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE))
 			m_pQueueLogSplitter->Initialize(m_pQueuePane);
-		else
-		{
+		else {
 			m_pQueuePane->Hide();
 			m_pQueueLogSplitter->Hide();
 		}
 		if (COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG))
 			m_pTopSplitter->SplitHorizontally(m_pStatusView, m_pBottomSplitter);
-		else
-		{
+		else {
 			m_pStatusView->Hide();
 			m_pTopSplitter->Initialize(m_pBottomSplitter);
 		}
@@ -422,8 +404,7 @@ CMainFrame::CMainFrame()
 
 	if (m_pQueueLogSplitter->IsShown())
 		m_pBottomSplitter->SplitHorizontally(m_pContextControl, m_pQueueLogSplitter);
-	else
-	{
+	else {
 		m_pQueueLogSplitter->Hide();
 		m_pBottomSplitter->Initialize(m_pContextControl);
 	}
@@ -437,20 +418,7 @@ CMainFrame::CMainFrame()
 	if (!RestoreSplitterPositions())
 		SetDefaultSplitterPositions();
 
-	std::vector<wxAcceleratorEntry> entries;
-	//entries[0].Set(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
-	for (int i = 0; i < 10; i++) {
-		tab_hotkey_ids[i] = wxNewId();
-		entries.push_back(wxAcceleratorEntry(wxACCEL_CMD, (int)'0' + i, tab_hotkey_ids[i]));
-	}
-	entries.push_back(wxAcceleratorEntry(wxACCEL_CMD | wxACCEL_SHIFT, 'O', m_comparisonToggleAcceleratorId));
-#ifdef __WXMAC__
-	for (auto it = keyboardCommands.begin(); it != keyboardCommands.end(); ++it) {
-		entries.push_back(wxAcceleratorEntry(wxACCEL_CMD, it->second.second, it->first));
-	}
-#endif
-	wxAcceleratorTable accel(entries.size(), &entries[0]);
-	SetAcceleratorTable(accel);
+	SetupKeyboardAccelerators();
 
 	ConnectNavigationHandler(m_pStatusView);
 	ConnectNavigationHandler(m_pQueuePane);
@@ -475,8 +443,7 @@ CMainFrame::~CMainFrame()
 #endif
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
-	if (pEditHandler)
-	{
+	if (pEditHandler) {
 		// This might leave temporary files behind,
 		// edit handler should clean them on next startup
 		pEditHandler->Release();
@@ -793,7 +760,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		if (!pState) {
 			return;
 		}
-		const CServer* pServer = pState ? pState->GetServer() : 0;
+		CServer const* pServer = pState->GetServer();
 
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 		if (!controls)
@@ -822,7 +789,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		else {
 			CBookmarksDialog dlg(this, controls->site_bookmarks->path, pServer);
 
-			res = dlg.Run(pState->GetLocalDir().GetPath(), pState->GetRemotePath());
+			res = dlg.Run();
 		}
 		if (res == wxID_OK) {
 			controls->site_bookmarks->bookmarks.clear();
@@ -926,22 +893,7 @@ void CMainFrame::OnEngineEvent(wxFzEvent &event)
 		case nId_listing:
 			{
 				auto const& listingNotification = static_cast<CDirectoryListingNotification const&>(*pNotification.get());
-
-				if (listingNotification.GetPath().empty())
-					pState->SetRemoteDir(0, false);
-				else {
-					std::shared_ptr<CDirectoryListing> pListing = std::make_shared<CDirectoryListing>();
-					if (listingNotification.Failed() ||
-						pState->m_pEngine->CacheLookup(listingNotification.GetPath(), *pListing) != FZ_REPLY_OK)
-					{
-						pListing = std::make_shared<CDirectoryListing>();
-						pListing->path = listingNotification.GetPath();
-						pListing->m_flags |= CDirectoryListing::listing_failed;
-						pListing->m_firstListTime = CMonotonicTime::Now();
-					}
-
-					pState->SetRemoteDir(pListing, listingNotification.Modified());
-				}
+				pState->m_pCommandQueue->ProcessDirectoryListing(listingNotification);
 			}
 			break;
 		case nId_asyncrequest:
@@ -1978,9 +1930,9 @@ bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site & data, bool newTab)
 	if (!ConnectToServer(data.m_server, data.m_remoteDir))
 		return false;
 
-	if (!data.m_localDir.empty()) {
-		CState *pState = CContextManager::Get()->GetCurrentContext();
-		if( pState ) {
+	CState *pState = CContextManager::Get()->GetCurrentContext();
+	if (pState) {
+		if (!data.m_localDir.empty()) {
 			bool set = pState->SetLocalDir(data.m_localDir, 0, false);
 
 			if (set && data.m_sync) {
@@ -1988,6 +1940,10 @@ bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site & data, bool newTab)
 				pState->SetSyncBrowse(true, data.m_remoteDir);
 			}
 		}
+
+		if (data.m_comparison && pState->GetComparisonManager()) {
+			pState->GetComparisonManager()->CompareListings();
+		}
 	}
 
 	SetBookmarksFromPath(data.m_path);
@@ -2396,14 +2352,17 @@ void CMainFrame::ProcessCommandLine()
 			server.SetLogonType(INTERACTIVE);
 
 		CServerPath path;
-		if (!server.ParseUrl(param, 0, _T(""), _T(""), error, path))
-		{
+		if (!server.ParseUrl(param, 0, _T(""), _T(""), error, path)) {
 			wxString str = _("Parameter not a valid URL");
 			str += _T("\n") + error;
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (server.GetLogonType() == ASK ||
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.GetLogonType() == NORMAL) {
+			server.SetLogonType(ASK);
+			CLoginManager::Get().RememberPassword(server);
+		}
+		else if (server.GetLogonType() == ASK ||
 			(server.GetLogonType() == INTERACTIVE && server.GetUser().empty()))
 		{
 			if (!CLoginManager::Get().GetPassword(server, false))
@@ -2713,3 +2672,28 @@ void CMainFrame::OnChildFocused(wxChildFocusEvent& event)
 	m_lastFocusedChild = event.GetWindow()->GetId();
 }
 #endif
+
+void CMainFrame::SetupKeyboardAccelerators()
+{
+	std::vector<wxAcceleratorEntry> entries;
+	for (int i = 0; i < 10; i++) {
+		tab_hotkey_ids[i] = wxNewId();
+		entries.emplace_back(wxACCEL_CMD, (int)'0' + i, tab_hotkey_ids[i]);
+	}
+	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'O', m_comparisonToggleAcceleratorId);
+	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
+#ifdef __WXMAC__
+	entries.emplace_back(wxACCEL_CMD, ',', XRCID("wxID_PREFERENCES"));
+
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->Cut(); }, 'X');
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->Copy(); }, 'C');
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->Paste(); }, 'V');
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->SelectAll(); }, 'A');
+
+	for (auto const& command : keyboardCommands) {
+		entries.emplace_back(wxACCEL_CMD, command.second.second, command.first);
+	}
+#endif
+	wxAcceleratorTable accel(entries.size(), &entries[0]);
+	SetAcceleratorTable(accel);
+}
