@@ -147,14 +147,12 @@ public:
 
 	void LoadQueue();
 	void LoadQueueFromXML();
-	void ImportQueue(TiXmlElement* pElement, bool updateSelections);
+	void ImportQueue(pugi::xml_node element, bool updateSelections);
 
 	virtual void InsertItem(CServerItem* pServerItem, CQueueItem* pItem);
 
 	virtual void CommitChanges();
 
-	void WriteToFile(TiXmlElement* pElement) const;
-
 	void ProcessNotification(CFileZillaEngine* pEngine, std::unique_ptr<CNotification>&& pNotification);
 
 	void RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString& newName, bool local);
@@ -232,7 +230,7 @@ protected:
 	void TryRefreshListings();
 	CServer m_last_refresh_server;
 	CServerPath m_last_refresh_path;
-	CMonotonicTime m_last_refresh_listing_time;
+	CMonotonicClock m_last_refresh_listing_time;
 
 	// Called from Process Reply.
 	// After a disconnect, check if there's another idle engine that
