@@ -124,10 +124,10 @@ void CStatusView::OnSize(wxSizeEvent &)
 
 void CStatusView::AddToLog(CLogmsgNotification const& notification)
 {
-	AddToLog(notification.msgType, notification.msg, wxDateTime::Now());
+	AddToLog(notification.msgType, notification.msg, CDateTime::Now());
 }
 
-void CStatusView::AddToLog(MessageType messagetype, const wxString& message, const wxDateTime& time)
+void CStatusView::AddToLog(MessageType messagetype, const wxString& message, CDateTime const& time)
 {
 	if (!m_shown) {
 		if (m_hiddenLines.size() >= MAX_LINECOUNT) {
@@ -183,10 +183,10 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, con
 		if (time != m_lastTime) {
 			m_lastTime = time;
 #ifndef __WXMAC__
-			m_lastTimeString = time.Format(_T("%H:%M:%S\t"));
+			m_lastTimeString = time.Format(_T("%H:%M:%S\t"), CDateTime::local);
 #else
 			// Tabs on OS X cannot be freely positioned
-			m_lastTimeString = time.Format(_T("%H:%M:%S "));
+			m_lastTimeString = time.Format(_T("%H:%M:%S "), CDateTime::local);
 #endif
 		}
 		prefix += m_lastTimeString;
@@ -255,8 +255,8 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, con
 void CStatusView::InitDefAttr()
 {
 	m_showTimestamps = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_TIMESTAMP) != 0;
-	m_lastTime = wxDateTime::Now();
-	m_lastTimeString = m_lastTime.Format(_T("%H:%M:%S\t"));
+	m_lastTime = CDateTime::Now();
+	m_lastTimeString = m_lastTime.Format(_T("%H:%M:%S\t"), CDateTime::local);
 
 	// Measure withs of all types
 	wxClientDC dc(this);
