@@ -8,14 +8,14 @@
 #include <wx/timer.h>
 
 class CFastTextCtrl;
-class CStatusView : public wxNavigationEnabled<wxWindow>
+class CStatusView final : public wxNavigationEnabled<wxWindow>
 {
 public:
 	CStatusView(wxWindow* parent, wxWindowID id);
 	virtual ~CStatusView();
 
 	void AddToLog(CLogmsgNotification const& pNotification);
-	void AddToLog(MessageType messagetype, const wxString& message, const wxDateTime& time);
+	void AddToLog(MessageType messagetype, const wxString& message, CDateTime const& time);
 
 	void InitDefAttr();
 
@@ -59,12 +59,12 @@ protected:
 	{
 		MessageType messagetype;
 		wxString message;
-		wxDateTime time;
+		CDateTime time;
 	};
 	std::list<t_line> m_hiddenLines;
 
 	bool m_showTimestamps{};
-	wxDateTime m_lastTime;
+	CDateTime m_lastTime;
 	wxString m_lastTimeString;
 };
 
