@@ -273,10 +273,10 @@ int CQueueItem::GetItemIndex() const
 
 CFileItem::CFileItem(CServerItem* parent, bool queued, bool download,
 					 const wxString& sourceFile, const wxString& targetFile,
-					 const CLocalPath& localPath, const CServerPath& remotePath, wxLongLong size)
+					 const CLocalPath& localPath, const CServerPath& remotePath, int64_t size)
 	: CQueueItem(parent)
 	, m_sourceFile(sourceFile)
-	, m_targetFile(targetFile.empty() ? CSparseOptional<wxString>() : CSparseOptional<wxString>(targetFile))
+	, m_targetFile(targetFile.empty() ? CSparseOptional<fzstring>() : CSparseOptional<fzstring>(to_fzstring(targetFile)))
 	, m_localPath(localPath)
 	, m_remotePath(remotePath)
 	, m_size(size)
@@ -316,32 +316,30 @@ QueuePriority CFileItem::GetPriority() const
 
 void CFileItem::SetActive(const bool active)
 {
-	if (active && !IsActive())
-	{
+	if (active && !IsActive()) {
 		AddChild(new CStatusItem);
 		flags |= flag_active;
 	}
-	else if (!active && IsActive())
-	{
+	else if (!active && IsActive()) {
 		CQueueItem* pItem = GetChild(0, false);
 		RemoveChild(pItem);
 		flags &= ~flag_active;
 	}
 }
 
-void CFileItem::SaveItem(TiXmlElement* pElement) const
+void CFileItem::SaveItem(pugi::xml_node& element) const
 {
-	if (m_edit != CEditHandler::none)
+	if (m_edit != CEditHandler::none || !element)
 		return;
 
-	TiXmlElement *file = pElement->LinkEndChild(new TiXmlElement("File"))->ToElement();
+	auto file = element.append_child("File");
 
 	AddTextElement(file, "LocalFile", m_localPath.GetPath() + GetLocalFile());
 	AddTextElement(file, "RemoteFile", GetRemoteFile());
 	AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
 	if (m_size != -1)
-		AddTextElement(file, "Size", m_size.ToString());
+		AddTextElement(file, "Size", m_size);
 	if (m_errorCount)
 		AddTextElement(file, "ErrorCount", m_errorCount);
 	if (m_priority != QueuePriority::normal)
@@ -363,7 +361,7 @@ bool CFileItem::TryRemoveAll()
 void CFileItem::SetTargetFile(wxString const& file)
 {
 	if (!file.empty() && file != m_sourceFile)
-		m_targetFile = CSparseOptional<wxString>(file);
+		m_targetFile = CSparseOptional<fzstring>(to_fzstring(file));
 	else
 		m_targetFile.clear();
 }
@@ -405,20 +403,17 @@ CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CServerPath& re
 {
 }
 
-void CFolderItem::SaveItem(TiXmlElement* pElement) const
+void CFolderItem::SaveItem(pugi::xml_node& element) const
 {
-	TiXmlElement *file = new TiXmlElement("Folder");
+	auto file = element.append_child("Folder");
 
 	if (Download())
 		AddTextElement(file, "LocalFile", GetLocalPath().GetPath() + GetLocalFile());
-	else
-	{
+	else {
 		AddTextElement(file, "RemoteFile", GetRemoteFile());
 		AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	}
 	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
-
-	pElement->LinkEndChild(file);
 }
 
 void CFolderItem::SetActive(const bool active)
@@ -571,8 +566,7 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 		return;
 
 	std::list<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
-	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter)
-	{
+	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
 		if (*iter != pItem)
 			continue;
 
@@ -584,29 +578,24 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 	wxASSERT(false);
 }
 
-void CServerItem::SaveItem(TiXmlElement* pElement) const
+void CServerItem::SaveItem(pugi::xml_node& element) const
 {
-	TiXmlElement *server = new TiXmlElement("Server");
+	auto server = element.append_child("Server");
 	SetServer(server, m_server);
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
+	for (auto iter = m_children.cbegin() + m_removed_at_front; iter != m_children.cend(); ++iter)
 		(*iter)->SaveItem(server);
-
-	pElement->LinkEndChild(server);
 }
 
-wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
+int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
 {
-	wxLongLong totalSize = 0;
-	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i)
-	{
-		for (int j = 0; j < 2; j++)
-		{
+	int64_t totalSize = 0;
+	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
+		for (int j = 0; j < 2; ++j) {
 			const std::list<CFileItem*>& fileList = m_fileList[j][i];
-			for (std::list<CFileItem*>::const_iterator iter = fileList.begin(); iter != fileList.end(); ++iter)
-			{
+			for (std::list<CFileItem*>::const_iterator iter = fileList.begin(); iter != fileList.end(); ++iter) {
 				const CFileItem* item = *iter;
-				wxLongLong size = item->GetSize();
+				int64_t size = item->GetSize();
 				if (size >= 0)
 					totalSize += size;
 				else
@@ -615,8 +604,7 @@ wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles
 		}
 	}
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
+	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		if ((*iter)->GetType() == QueueItemType::File ||
 			(*iter)->GetType() == QueueItemType::Folder)
 			queuedFiles++;
@@ -669,7 +657,7 @@ void CServerItem::DetachChildren()
 	m_maxCachedIndex = -1;
 	m_removed_at_front = 0;
 
-	for (int i = 0; i < 2; i++)
+	for (int i = 0; i < 2; ++i)
 		for (int j = 0; j < static_cast<int>(QueuePriority::count); j++)
 			m_fileList[i][j].clear();
 }
@@ -677,8 +665,7 @@ void CServerItem::DetachChildren()
 void CServerItem::SetPriority(QueuePriority priority)
 {
 	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		if ((*iter)->GetType() == QueueItemType::File)
 			((CFileItem*)(*iter))->SetPriorityRaw(priority);
 		else
@@ -737,6 +724,7 @@ EVT_CHAR(CQueueViewBase::OnChar)
 EVT_LIST_COL_END_DRAG(wxID_ANY, CQueueViewBase::OnEndColumnDrag)
 EVT_TIMER(wxID_ANY, CQueueViewBase::OnTimer)
 EVT_KEY_DOWN(CQueueViewBase::OnKeyDown)
+EVT_MENU(XRCID("ID_EXPORT"), CQueueViewBase::OnExport)
 END_EVENT_TABLE()
 
 CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
@@ -775,17 +763,14 @@ CQueueViewBase::~CQueueViewBase()
 		delete *iter;
 }
 
-CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item)
+CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item) const
 {
-	std::vector<CServerItem*>::iterator iter;
-	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
+	for (auto iter = m_serverList.cbegin(); iter != m_serverList.cend(); ++iter) {
 		if (!item)
 			return *iter;
 
 		unsigned int count = (*iter)->GetChildrenCount(true);
-		if (item > count)
-		{
+		if (item > count) {
 			item -= count + 1;
 			continue;
 		}
@@ -865,9 +850,9 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 				return pFileItem->GetRemotePath().FormatFilename(pFileItem->GetRemoteFile());
 			case colSize:
 				{
-					const wxLongLong& size = pFileItem->GetSize();
+					auto const& size = pFileItem->GetSize();
 					if (size >= 0)
-						return CSizeFormat::Format(size.GetValue());
+						return CSizeFormat::Format(size);
 					else
 						return _T("?");
 				}
@@ -1072,16 +1057,14 @@ void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 		return;
 #endif
 
-	std::list<int> itemsToSelect;
+	std::deque<int> itemsToSelect;
 
 	// Go through all selected items
 	int item = GetNextItem(added - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
-	while (item != -1)
-	{
+	while (item != -1) {
 		// Select new items preceding to current one
-		while (!itemsToSelect.empty() && itemsToSelect.front() < item)
-		{
+		while (!itemsToSelect.empty() && itemsToSelect.front() < item) {
 			SetItemState(itemsToSelect.front(), wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 			itemsToSelect.pop_front();
 		}
@@ -1094,8 +1077,9 @@ void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	for (std::list<int>::const_iterator iter = itemsToSelect.begin(); iter != itemsToSelect.end(); ++iter)
-		SetItemState(*iter, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+	for (auto const& sel : itemsToSelect) {
+		SetItemState(sel, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+	}
 }
 
 void CQueueViewBase::UpdateSelections_ItemRemoved(int removed)
@@ -1151,15 +1135,13 @@ void CQueueViewBase::UpdateSelections_ItemRangeRemoved(int removed, int count)
 
 	SetItemState(removed, 0, wxLIST_STATE_SELECTED);
 
-	std::list<int> itemsToUnselect;
+	std::deque<int> itemsToUnselect;
 
 	int item = GetNextItem(removed - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
-	while (item != -1)
-	{
+	while (item != -1) {
 		// Unselect new items preceding to current one
-		while (!itemsToUnselect.empty() && itemsToUnselect.front() < item - count)
-		{
+		while (!itemsToUnselect.empty() && itemsToUnselect.front() < item - count) {
 			SetItemState(itemsToUnselect.front(), 0, wxLIST_STATE_SELECTED);
 			itemsToUnselect.pop_front();
 		}
@@ -1175,8 +1157,9 @@ void CQueueViewBase::UpdateSelections_ItemRangeRemoved(int removed, int count)
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	for (std::list<int>::const_iterator iter = itemsToUnselect.begin(); iter != itemsToUnselect.end(); ++iter)
-		SetItemState(*iter, 0, wxLIST_STATE_SELECTED);
+	for (auto const& unsel : itemsToUnselect) {
+		SetItemState(unsel, 0, wxLIST_STATE_SELECTED);
+	}
 }
 
 void CQueueViewBase::AddQueueColumn(ColumnId id)
@@ -1255,15 +1238,13 @@ void CQueueViewBase::CommitChanges()
 
 void CQueueViewBase::DisplayNumberQueuedFiles()
 {
-	if (m_filecount_delay_timer.IsRunning())
-	{
+	if (m_filecount_delay_timer.IsRunning()) {
 		m_fileCountChanged = true;
 		return;
 	}
 
 	wxString str;
-	if (m_fileCount > 0)
-	{
+	if (m_fileCount > 0) {
 		if (!m_folderScanCount)
 			str.Printf(m_title + _T(" (%d)"), m_fileCount);
 		else
@@ -1294,13 +1275,11 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 		m_insertionStart = newIndex;
 	m_insertionCount++;
 
-	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-	{
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		m_fileCount++;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType::FolderScan)
-	{
+	else if (pItem->GetType() == QueueItemType::FolderScan) {
 		m_folderScanCount++;
 		m_folderScanCountChanged = true;
 	}
@@ -1308,14 +1287,12 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 
 bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount /*=true*/, bool updateSelections /*=true*/)
 {
-	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-	{
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		wxASSERT(m_fileCount > 0);
 		m_fileCount--;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType::FolderScan)
-	{
+	else if (pItem->GetType() == QueueItemType::FolderScan) {
 		wxASSERT(m_folderScanCount > 0);
 		m_folderScanCount--;
 		m_folderScanCountChanged = true;
@@ -1334,11 +1311,9 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 	bool didRemoveParent;
 
 	int oldCount = m_itemCount;
-	if (!topLevelItem->GetChild(0))
-	{
+	if (!topLevelItem->GetChild(0)) {
 		std::vector<CServerItem*>::iterator iter;
-		for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-		{
+		for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
 			if (*iter == topLevelItem)
 				break;
 		}
@@ -1355,8 +1330,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 
 		didRemoveParent = true;
 	}
-	else
-	{
+	else {
 		count -= topLevelItem->GetChildrenCount(true);
 
 		if (updateSelections)
@@ -1369,8 +1343,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 		didRemoveParent = false;
 	}
 
-	if (updateItemCount)
-	{
+	if (updateItemCount) {
 		if (m_fileCountChanged || m_folderScanCountChanged)
 			DisplayNumberQueuedFiles();
 		if (oldCount > m_itemCount)
@@ -1447,8 +1420,9 @@ void CQueueViewBase::OnTimer(wxTimerEvent& event)
 		return;
 	}
 
-	if (m_fileCountChanged || m_folderScanCountChanged)
+	if (m_fileCountChanged || m_folderScanCountChanged) {
 		DisplayNumberQueuedFiles();
+	}
 }
 
 void CQueueViewBase::OnKeyDown(wxKeyEvent& event)
@@ -1464,6 +1438,35 @@ void CQueueViewBase::OnKeyDown(wxKeyEvent& event)
 		event.Skip();
 }
 
+void CQueueViewBase::WriteToFile(pugi::xml_node element) const
+{
+	auto queue = element.child("Queue");
+	if (!queue) {
+		queue = element.append_child("Queue");
+	}
+
+	for (std::vector<CServerItem*>::const_iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
+		(*iter)->SaveItem(queue);
+}
+
+void CQueueViewBase::OnExport(wxCommandEvent&)
+{
+	wxFileDialog dlg(m_parent, _("Select file for exported queue"), wxString(),
+		_T("FileZilla.xml"), _T("XML files (*.xml)|*.xml"),
+		wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+
+	if (dlg.ShowModal() != wxID_OK)
+		return;
+
+	CXmlFile xml(dlg.GetPath());
+
+	auto exportRoot = xml.CreateEmpty();
+
+	WriteToFile(exportRoot);
+
+	xml.Save(true);
+}
+
 // ------
 // CQueue
 // ------
