@@ -31,7 +31,7 @@ enum class TransferDirection
 	upload
 };
 
-class TiXmlElement;
+namespace pugi { class xml_node; }
 class CQueueItem
 {
 public:
@@ -51,12 +51,12 @@ public:
 	CQueueItem* GetTopLevelItem();
 	const CQueueItem* GetTopLevelItem() const;
 	int GetItemIndex() const; // Return the visible item index relative to the topmost parent item.
-	virtual void SaveItem(TiXmlElement*) const {}
+	virtual void SaveItem(pugi::xml_node&) const {}
 
 	virtual QueueItemType GetType() const = 0;
 
-	wxDateTime GetTime() const { return m_time; }
-	void UpdateTime() { m_time = wxDateTime::UNow(); }
+	CDateTime GetTime() const { return m_time; }
+	void UpdateTime() { m_time = CDateTime::Now(); }
 
 	const std::vector<CQueueItem*>& GetChildren() const { return m_children; }
 	int GetRemovedAtFront() const { return m_removed_at_front; }
@@ -78,7 +78,7 @@ protected:
 
 	friend class CServerItem;
 
-	wxDateTime m_time;
+	CDateTime m_time;
 
 private:
 	std::vector<CQueueItem*> m_children;
@@ -104,12 +104,12 @@ public:
 
 	CFileItem* GetIdleChild(bool immadiateOnly, TransferDirection direction);
 	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true); // Removes a child item with is somewhere in the tree of children
-	wxLongLong GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const;
+	int64_t GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const;
 
 	void QueueImmediateFiles();
 	void QueueImmediateFile(CFileItem* pItem);
 
-	virtual void SaveItem(TiXmlElement* pElement) const;
+	virtual void SaveItem(pugi::xml_node& element) const;
 
 	void SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction);
 
@@ -150,12 +150,12 @@ public:
 	void SetPriorityRaw(QueuePriority priority);
 	QueuePriority GetPriority() const;
 
-	const wxString& GetLocalFile() const { return !Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
-	const wxString& GetRemoteFile() const { return Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
-	const wxString& GetSourceFile() const { return m_sourceFile; }
-	CSparseOptional<wxString> const& GetTargetFile() const { return m_targetFile; }
-	const CLocalPath& GetLocalPath() const { return m_localPath; }
-	const CServerPath& GetRemotePath() const { return m_remotePath; }
+	fzstring const& GetLocalFile() const { return !Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
+	fzstring const& GetRemoteFile() const { return Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
+	fzstring const& GetSourceFile() const { return m_sourceFile; }
+	CSparseOptional<fzstring> const& GetTargetFile() const { return m_targetFile; }
+	CLocalPath const& GetLocalPath() const { return m_localPath; }
+	CServerPath const& GetRemotePath() const { return m_remotePath; }
 	int64_t GetSize() const { return m_size; }
 	void SetSize(int64_t size) { m_size = size; }
 	inline bool Download() const { return flags & flag_download; }
@@ -183,7 +183,7 @@ public:
 	bool IsActive() const { return (flags & flag_active) != 0; }
 	virtual void SetActive(bool active);
 
-	virtual void SaveItem(TiXmlElement* pElement) const;
+	virtual void SaveItem(pugi::xml_node& element) const;
 
 	virtual bool TryRemoveAll(); // Removes a inactive childrens, queues active children for removal.
 								 // Returns true if item can be removed itself
@@ -255,8 +255,8 @@ public:
 	}
 
 protected:
-	wxString const m_sourceFile;
-	CSparseOptional<wxString> m_targetFile;
+	fzstring const m_sourceFile;
+	CSparseOptional<fzstring> m_targetFile;
 	CLocalPath const m_localPath;
 	CServerPath const m_remotePath;
 	int64_t m_size{};
@@ -270,7 +270,7 @@ public:
 
 	virtual QueueItemType GetType() const { return QueueItemType::Folder; }
 
-	virtual void SaveItem(TiXmlElement* pElement) const;
+	virtual void SaveItem(pugi::xml_node& element) const;
 
 	virtual void SetActive(bool active);
 };
@@ -354,6 +354,8 @@ public:
 
 	int GetFileCount() const { return m_fileCount; }
 
+	void WriteToFile(pugi::xml_node element) const;
+
 protected:
 
 	void CreateColumns(std::list<ColumnId> const& extraColumns = std::list<ColumnId>());
@@ -363,7 +365,7 @@ protected:
 	CServerItem* GetServerItem(const CServer& server);
 
 	// Gets item with given index
-	CQueueItem* GetQueueItem(unsigned int item);
+	CQueueItem* GetQueueItem(unsigned int item) const;
 
 	// Get index for given queue item
 	int GetItemIndex(const CQueueItem* item);
@@ -412,6 +414,7 @@ protected:
 	void OnEndColumnDrag(wxListEvent& event);
 	void OnTimer(wxTimerEvent& event);
 	void OnKeyDown(wxKeyEvent& event);
+	void OnExport(wxCommandEvent&);
 };
 
 class CQueueView;
