@@ -7,7 +7,6 @@
 #include "queue.h"
 #include "filezillaapp.h"
 #include "recursive_operation.h"
-#include "statusbar.h"
 #include "local_filesys.h"
 #include "listingcomparison.h"
 
@@ -23,23 +22,20 @@ CContextManager* CContextManager::Get()
 	return &m_the_context_manager;
 }
 
-CState* CContextManager::CreateState(CMainFrame* pMainFrame)
+CState* CContextManager::CreateState(CMainFrame &mainFrame)
 {
-	wxASSERT(pMainFrame);
-
-	CState* pState = new CState(pMainFrame);
+	CState* pState = new CState(mainFrame);
 
 	m_contexts.push_back(pState);
 
-	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0, false);
+	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0);
 
 	return pState;
 }
 
 void CContextManager::DestroyState(CState* pState)
 {
-	for (unsigned int i = 0; i < m_contexts.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_contexts.size(); ++i) {
 		if (m_contexts[i] != pState)
 			continue;
 
@@ -50,13 +46,13 @@ void CContextManager::DestroyState(CState* pState)
 		{
 			if (i >= m_contexts.size())
 				m_current_context--;
-			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 		}
 
 		break;
 	}
 
-	NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
+	NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0);
 	delete pState;
 }
 
@@ -71,26 +67,26 @@ void CContextManager::SetCurrentContext(CState* pState)
 			continue;
 
 		m_current_context = i;
-		NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+		NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 	}
 }
 
 void CContextManager::DestroyAllStates()
 {
 	m_current_context = -1;
-	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0, false);
+	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 
 	while (!m_contexts.empty())
 	{
 		CState* pState = m_contexts.back();
 		m_contexts.pop_back();
 
-		NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0, false);
+		NotifyHandlers(pState, STATECHANGE_REMOVECONTEXT, _T(""), 0);
 		delete pState;
 	}
 }
 
-void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only, bool blockable)
+void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
@@ -104,7 +100,6 @@ void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_state
 
 	t_handler handler;
 	handler.pHandler = pHandler;
-	handler.blockable = blockable;
 	handler.current_only = current_only;
 	handlers.push_back(handler);
 }
@@ -142,15 +137,12 @@ size_t CContextManager::HandlerCount(t_statechange_notifications notification) c
 	return m_handlers[notification].size();
 }
 
-void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2, bool blocked)
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
-		if (blocked && handler.blockable)
-			continue;
-
 		if (handler.current_only && pState != GetCurrentContext())
 			continue;
 
@@ -180,19 +172,11 @@ void CContextManager::NotifyGlobalHandlers(enum t_statechange_notifications noti
 	}
 }
 
-CState::CState(CMainFrame* pMainFrame)
+CState::CState(CMainFrame &mainFrame)
+	: m_mainFrame(mainFrame)
 {
-	memset(m_blocked, 0, sizeof(m_blocked));
-
-	m_pMainFrame = pMainFrame;
-
-	m_pDirectoryListing = 0;
-	m_pServer = 0;
 	m_title = _("Not connected");
-	m_successful_connect = 0;
 
-	m_pEngine = 0;
-	m_pCommandQueue = 0;
 	m_pComparisonManager = new CComparisonManager(this);
 
 	m_pRecursiveOperation = new CRecursiveOperation(this);
@@ -201,9 +185,6 @@ CState::CState(CMainFrame* pMainFrame)
 	m_sync_browse.compare = false;
 
 	m_localDir.SetPath(CLocalPath::path_separator);
-
-	m_pCertificate = 0;
-	m_pSftpEncryptionInfo = 0;
 }
 
 CState::~CState()
@@ -271,7 +252,7 @@ bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPr
 				return false;
 			SetSyncBrowse(false);
 		}
-		else if (!IsRemoteIdle()) {
+		else if (!IsRemoteIdle(true)) {
 			wxString msg(_("A remote operation is in progress and synchronized browsing is enabled.\nDisable synchronized browsing and continue changing the local directory?"));
 			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 				return false;
@@ -326,16 +307,15 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 		if (modified)
 			return false;
 
-		if (m_pDirectoryListing)
-		{
+		if (m_pDirectoryListing) {
 			m_pDirectoryListing = 0;
-			NotifyHandlers(STATECHANGE_REMOTE_DIR);
+			NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &modified);
 		}
 		m_previouslyVisitedRemoteSubdir = _T("");
 		return true;
 	}
 
-	wxASSERT(pDirectoryListing->m_firstListTime.IsValid());
+	wxASSERT(pDirectoryListing->m_firstListTime);
 
 	if (pDirectoryListing && m_pDirectoryListing &&
 		pDirectoryListing->path == m_pDirectoryListing->path.GetParent())
@@ -361,15 +341,11 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 	m_pDirectoryListing = pDirectoryListing;
 
-	if (!modified)
-		NotifyHandlers(STATECHANGE_REMOTE_DIR);
-	else
-		NotifyHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
+	NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &modified);
 
 	if (m_sync_browse.is_changing && !modified) {
 		m_sync_browse.is_changing = false;
-		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false))
-		{
+		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false)) {
 			SetSyncBrowse(false);
 			wxString msg = wxString::Format(_("Current remote directory (%s) is not below the synchronization root (%s).\nSynchronized browsing has been disabled."),
 					m_pDirectoryListing->path.GetPath(),
@@ -466,7 +442,7 @@ void CState::LocalDirCreated(const CLocalPath& path)
 	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, next_segment);
 }
 
-void CState::SetServer(const CServer* server)
+void CState::SetServer(const CServer* server, CServerPath const& path)
 {
 	if (m_pServer) {
 		if (server && *server == *m_pServer &&
@@ -483,8 +459,12 @@ void CState::SetServer(const CServer* server)
 		m_pSftpEncryptionInfo.reset();
 	}
 	if (server) {
-		if (m_last_server != *server)
+		if (!path.empty()) {
+			m_last_path = path;
+		}
+		else if (m_last_server != *server) {
 			m_last_path.clear();
+		}
 		m_last_server = *server;
 
 		m_pServer = new CServer(*server);
@@ -527,7 +507,7 @@ bool CState::Connect(const CServer& server, const CServerPath& path /*=CServerPa
 	m_pCommandQueue->ProcessCommand(new CConnectCommand(server));
 	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
 
-	SetServer(&server);
+	SetServer(&server, path);
 
 	return true;
 }
@@ -555,10 +535,10 @@ bool CState::CreateEngine()
 	if (m_pEngine)
 		return true;
 
-	m_pEngine = new CFileZillaEngine(m_pMainFrame->GetEngineContext());
-	m_pEngine->Init(m_pMainFrame);
+	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext());
+	m_pEngine->Init(&m_mainFrame);
 
-	m_pCommandQueue = new CCommandQueue(m_pEngine, m_pMainFrame, this);
+	m_pCommandQueue = new CCommandQueue(m_pEngine, &m_mainFrame, this);
 
 	return true;
 }
@@ -571,7 +551,7 @@ void CState::DestroyEngine()
 	m_pEngine = 0;
 }
 
-void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool blockable /*=true*/)
+void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
 {
 	wxASSERT(pHandler);
 	wxASSERT(pHandler->m_pState == this);
@@ -588,7 +568,6 @@ void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_no
 
 	t_handler handler;
 	handler.pHandler = pHandler;
-	handler.blockable = blockable;
 	handlers.push_back(handler);
 }
 
@@ -619,46 +598,16 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	}
 }
 
-void CState::BlockHandlers(enum t_statechange_notifications notification)
-{
-	wxASSERT(notification != STATECHANGE_MAX);
-
-	if (notification == STATECHANGE_NONE)
-	{
-		for (int i = 0; i < STATECHANGE_MAX; i++)
-			m_blocked[i] = true;
-	}
-	else
-		m_blocked[notification] = true;
-}
-
-void CState::UnblockHandlers(enum t_statechange_notifications notification)
-{
-	wxASSERT(notification != STATECHANGE_MAX);
-
-	if (notification == STATECHANGE_NONE)
-	{
-		for (int i = 0; i < STATECHANGE_MAX; i++)
-			m_blocked[i] = false;
-	}
-	else
-		m_blocked[notification] = false;
-
-}
-
-void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
-		if (m_blocked[notification] && handler.blockable)
-			continue;
-
 		handler.pHandler->OnStateChange(this, notification, data, data2);
 	}
 
-	CContextManager::Get()->NotifyHandlers(this, notification, data, data2, m_blocked[notification]);
+	CContextManager::Get()->NotifyHandlers(this, notification, data, data2);
 }
 
 CStateEventHandler::CStateEventHandler(CState* pState)
@@ -705,8 +654,8 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 		{
 			wxString localFile;
 			const CLocalPath localPath(files[i], &localFile);
-			m_pMainFrame->GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, *m_pServer, size);
-			m_pMainFrame->GetQueue()->QueueFile_Finish(!queueOnly);
+			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, *m_pServer, size);
+			m_mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == CLocalFileSystem::dir)
 		{
@@ -715,7 +664,7 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 			{
 				CServerPath target = path;
 				target.AddSegment(localPath.GetLastSegment());
-				m_pMainFrame->GetQueue()->QueueFolder(queueOnly, false, localPath, target, *m_pServer);
+				m_mainFrame.GetQueue()->QueueFolder(queueOnly, false, localPath, target, *m_pServer);
 			}
 		}
 	}
@@ -752,7 +701,7 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	op.pFrom = from;
 	op.pTo = to;
 	op.wFunc = copy ? FO_COPY : FO_MOVE;
-	op.hwnd = (HWND)m_pMainFrame->GetHandle();
+	op.hwnd = (HWND)m_mainFrame.GetHandle();
 	SHFileOperation(&op);
 
 	delete [] to;
@@ -880,20 +829,18 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 			hasFiles = true;
 	}
 
-	if (hasDirs)
-	{
+	if (hasDirs) {
 		if (!IsRemoteConnected() || !IsRemoteIdle())
 			return false;
 	}
 
 	if (hasFiles)
-		m_pMainFrame->GetQueue()->QueueFiles(queueOnly, path, *pRemoteDataObject);
+		m_mainFrame.GetQueue()->QueueFiles(queueOnly, path, *pRemoteDataObject);
 
 	if (!hasDirs)
 		return true;
 
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		if (!iter->dir)
 			continue;
 
@@ -919,24 +866,20 @@ bool CState::IsRemoteConnected() const
 	return m_pServer != 0;
 }
 
-bool CState::IsRemoteIdle() const
+bool CState::IsRemoteIdle(bool ignore_recursive) const
 {
-	if (m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+	if (!ignore_recursive && m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
 		return false;
 
 	if (!m_pCommandQueue)
 		return true;
 
-	return m_pCommandQueue->Idle();
+	return m_pCommandQueue->Idle(ignore_recursive ? CCommandQueue::normal : CCommandQueue::any);
 }
 
 void CState::ListingFailed(int error)
 {
 	m_sync_browse.is_changing = false;
-
-	// Let the recursive operation handler know if a LIST command failed,
-	// so that it may issue the next command in recursive operations.
-	m_pRecursiveOperation->ListingFailed(error);
 }
 
 void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
@@ -951,18 +894,15 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 	if (!m_pServer || !m_pCommandQueue)
 		return false;
 
-	if (!m_sync_browse.local_root.empty())
-	{
+	if (!m_sync_browse.local_root.empty()) {
 		CServerPath p(path);
-		if (!subdir.empty() && !p.ChangePath(subdir))
-		{
+		if (!subdir.empty() && !p.ChangePath(subdir)) {
 			wxString msg = wxString::Format(_("Could not get full remote path."));
 			wxMessageBoxEx(msg, _("Synchronized browsing"));
 			return false;
 		}
 
-		if (p != m_sync_browse.remote_root && !p.IsSubdirOf(m_sync_browse.remote_root, false))
-		{
+		if (p != m_sync_browse.remote_root && !p.IsSubdirOf(m_sync_browse.remote_root, false)) {
 			wxString msg = wxString::Format(_("The remote directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the remote directory?"),
 					p.GetPath(),
 					m_sync_browse.remote_root.GetPath());
@@ -970,33 +910,28 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 				return false;
 			SetSyncBrowse(false);
 		}
-		else if (!IsRemoteIdle() && !ignore_busy)
-		{
+		else if (!IsRemoteIdle(true) && !ignore_busy) {
 			wxString msg(_("Another remote operation is already in progress, cannot change directory now."));
 			wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_EXCLAMATION);
 			return false;
 		}
-		else
-		{
+		else {
 			wxString error;
 			CLocalPath local_path = GetSynchronizedDirectory(p);
-			if (local_path.empty())
-			{
+			if (local_path.empty()) {
 				wxString msg = wxString::Format(_("Could not obtain corresponding local directory for the remote directory '%s'.\nDisable synchronized browsing and continue changing the remote directory?"),
 					p.GetPath());
 				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 					return false;
 				SetSyncBrowse(false);
 			}
-			else if (!local_path.Exists(&error))
-			{
+			else if (!local_path.Exists(&error)) {
 				wxString msg = error + _T("\n") + _("Disable synchronized browsing and continue changing the remote directory?");
 				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 					return false;
 				SetSyncBrowse(false);
 			}
-			else
-			{
+			else {
 				m_sync_browse.is_changing = true;
 				m_sync_browse.compare = m_pComparisonManager->IsComparing();
 			}
@@ -1097,7 +1032,7 @@ bool CState::RefreshRemote()
 	if (!m_pCommandQueue)
 		return false;
 
-	if (!IsRemoteConnected() || !IsRemoteIdle())
+	if (!IsRemoteConnected() || !IsRemoteIdle(true))
 		return false;
 
 	return ChangeRemoteDir(GetRemotePath(), _T(""), LIST_FLAG_REFRESH);
@@ -1118,13 +1053,13 @@ bool CState::GetSecurityInfo(CSftpEncryptionNotification *& pInfo)
 void CState::SetSecurityInfo(CCertificateNotification const& info)
 {
 	m_pSftpEncryptionInfo.reset();
-	m_pCertificate = make_unique<CCertificateNotification>(info);
+	m_pCertificate = std::make_unique<CCertificateNotification>(info);
 	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
 
 void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 {
 	m_pCertificate.reset();
-	m_pSftpEncryptionInfo = make_unique<CSftpEncryptionNotification>(info);
+	m_pSftpEncryptionInfo = std::make_unique<CSftpEncryptionNotification>(info);
 	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
