@@ -4,6 +4,7 @@
 #include "dialogex.h"
 #include "ipcmutex.h"
 #include "Options.h"
+#include "timeformatting.h"
 #include "xrc_helper.h"
 
 #include <wx/scrolwin.h>
@@ -27,12 +28,12 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 {
 	bool warning = false;
 	if (cert.GetActivationTime().IsValid()) {
-		if (cert.GetActivationTime() > wxDateTime::Now()) {
-			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), cert.GetActivationTime().FormatDate()));
+		if (cert.GetActivationTime() > CDateTime::Now()) {
+			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), CTimeFormat::Format(cert.GetActivationTime())));
 			warning = true;
 		}
 		else
-			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), cert.GetActivationTime().FormatDate());
+			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), CTimeFormat::Format(cert.GetActivationTime()));
 	}
 	else {
 		warning = true;
@@ -40,12 +41,12 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 	}
 
 	if (cert.GetExpirationTime().IsValid()) {
-		if (cert.GetExpirationTime() < wxDateTime::Now()) {
-			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), cert.GetExpirationTime().FormatDate()));
+		if (cert.GetExpirationTime() < CDateTime::Now()) {
+			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), CTimeFormat::Format(cert.GetExpirationTime())));
 			warning = true;
 		}
 		else
-			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), cert.GetExpirationTime().FormatDate());
+			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), CTimeFormat::Format(cert.GetExpirationTime()));
 	}
 	else {
 		warning = true;
@@ -407,55 +408,54 @@ void CVerifyCertDialog::LoadTrustedCerts()
 		return;
 	}
 
-	TiXmlElement* pElement = m_xmlFile.Load();
-	if (!pElement) {
+	auto element = m_xmlFile.Load();
+	if (!element) {
 		return;
 	}
 
 	m_trustedCerts.clear();
 
-	if (!(pElement = pElement->FirstChildElement("TrustedCerts")))
+	if (!(element = element.child("TrustedCerts")))
 		return;
 
 	bool modified = false;
 
-	TiXmlElement* pCert = pElement->FirstChildElement("Certificate");
-	while (pCert) {
-		wxString value = GetTextElement(pCert, "Data");
+	auto cert = element.child("Certificate");
+	while (cert) {
+		wxString value = GetTextElement(cert, "Data");
 
-		TiXmlElement* pRemove = 0;
+		pugi::xml_node remove;
 
 		t_certData data;
 		if (value.empty() || !(data.data = ConvertStringToHex(value, data.len)))
-			pRemove = pCert;
+			remove = cert;
 
-		data.host = GetTextElement(pCert, "Host");
-		data.port = GetTextElementInt(pCert, "Port");
+		data.host = GetTextElement(cert, "Host");
+		data.port = GetTextElementInt(cert, "Port");
 		if (data.host.empty() || data.port < 1 || data.port > 65535)
-			pRemove = pCert;
+			remove = cert;
 
-		wxLongLong activationTime = GetTextElementLongLong(pCert, "ActivationTime", 0);
+		int64_t activationTime = GetTextElementInt(cert, "ActivationTime", 0);
 		if (activationTime == 0 || activationTime > wxDateTime::GetTimeNow())
-			pRemove = pCert;
+			remove = cert;
 
-		wxLongLong expirationTime = GetTextElementLongLong(pCert, "ExpirationTime", 0);
+		int64_t expirationTime = GetTextElementInt(cert, "ExpirationTime", 0);
 		if (expirationTime == 0 || expirationTime < wxDateTime::GetTimeNow())
-			pRemove = pCert;
+			remove = cert;
 
 		if (IsTrusted(data.host, data.port, data.data, data.len, true)) // Weed out duplicates
-			pRemove = pCert;
+			remove = cert;
 
-		if (!pRemove)
+		if (!remove)
 			m_trustedCerts.push_back(data);
 		else
 			delete [] data.data;
 
-		pCert = pCert->NextSiblingElement("Certificate");
+		cert = cert.next_sibling("Certificate");
 
-		if (pRemove)
-		{
+		if (remove) {
 			modified = true;
-			pElement->RemoveChild(pRemove);
+			element.remove_child(remove);
 		}
 	}
 
@@ -488,27 +488,21 @@ void CVerifyCertDialog::SetPermanentlyTrusted(CCertificateNotification const& no
 		return;
 	}
 
-	TiXmlElement* pElement = m_xmlFile.GetElement();
-	if (!pElement) {
+	auto element = m_xmlFile.GetElement();
+	if (!element) {
 		return;
 	}
 
-	TiXmlElement* pCerts = pElement->FirstChildElement("TrustedCerts");
-	if (!pCerts)
-		pCerts = pElement->LinkEndChild(new TiXmlElement("TrustedCerts"))->ToElement();
+	auto certs = element.child("TrustedCerts");
+	if (!certs)
+		certs = element.append_child("TrustedCerts");
 
-	TiXmlElement* pCert = pCerts->LinkEndChild(new TiXmlElement("Certificate"))->ToElement();
-
-	AddTextElement(pCert, "Data", ConvertHexToString(data, len));
-
-	wxLongLong time = certificate.GetActivationTime().GetTicks();
-	AddTextElement(pCert, "ActivationTime", time.ToString());
-
-	time = certificate.GetExpirationTime().GetTicks();
-	AddTextElement(pCert, "ExpirationTime", time.ToString());
-
-	AddTextElement(pCert, "Host", notification.GetHost());
-	AddTextElement(pCert, "Port", notification.GetPort());
+	auto xCert = certs.append_child("Certificate");
+	AddTextElement(xCert, "Data", ConvertHexToString(data, len));
+	AddTextElement(xCert, "ActivationTime", static_cast<int64_t>(certificate.GetActivationTime().GetTimeT()));
+	AddTextElement(xCert, "ExpirationTime", static_cast<int64_t>(certificate.GetExpirationTime().GetTimeT()));
+	AddTextElement(xCert, "Host", notification.GetHost());
+	AddTextElement(xCert, "Port", notification.GetPort());
 
 	m_xmlFile.Save(true);
 }
