@@ -14,7 +14,7 @@
 #include "putty.h"
 #include "ssh.h"
 
-void modalfatalbox(char *fmt, ...)
+void modalfatalbox(const char *fmt, ...)
 {
     va_list ap;
     char* str;
@@ -28,7 +28,7 @@ void modalfatalbox(char *fmt, ...)
     cleanup_exit(1);
 }
 
-void nonfatal(char *fmt, ...)
+void nonfatal(const char *fmt, ...)
 {
 	va_list ap;
 	char* str;
@@ -137,7 +137,8 @@ int main(int argc, char **argv)
 		fzprintf(sftpReply, "2");
 		intype = SSH_KEYTYPE_UNOPENABLE;
 		break;
-	    case SSH_KEYTYPE_OPENSSH:
+	    case SSH_KEYTYPE_OPENSSH_PEM:
+	    case SSH_KEYTYPE_OPENSSH_NEW:
 	    case SSH_KEYTYPE_SSHCOM:
 		fzprintf(sftpReply, "1");
 		break;
@@ -147,7 +148,7 @@ int main(int argc, char **argv)
 	{
 	    if (intype == SSH_KEYTYPE_UNOPENABLE)
 	    {
-		fzprintf(sftpError, "No keyfile opened");
+		fzprintf(sftpError, "No key file opened");
 		continue;
 	    }
 	    
@@ -164,7 +165,7 @@ int main(int argc, char **argv)
 	{
 	    if (intype == SSH_KEYTYPE_UNOPENABLE)
 	    {
-		fzprintf(sftpError, "No keyfile opened");
+		fzprintf(sftpError, "No key file opened");
 		continue;
 	    }
 	    if (origcomment)
@@ -181,7 +182,7 @@ int main(int argc, char **argv)
 
 	    if (intype == SSH_KEYTYPE_UNOPENABLE)
 	    {
-		fzprintf(sftpError, "No keyfile opened");
+		fzprintf(sftpError, "No key file opened");
 		continue;
 	    }
 
@@ -208,7 +209,7 @@ int main(int argc, char **argv)
 
 	    if (intype == SSH_KEYTYPE_UNOPENABLE)
 	    {
-		fzprintf(sftpError, "No keyfile opened");
+		fzprintf(sftpError, "No key file opened");
 		continue;
 	    }
 
@@ -244,7 +245,8 @@ int main(int argc, char **argv)
 		    error = "unknown error";
 		break;
 
-	    case SSH_KEYTYPE_OPENSSH:
+	    case SSH_KEYTYPE_OPENSSH_PEM:
+	    case SSH_KEYTYPE_OPENSSH_NEW:
 	    case SSH_KEYTYPE_SSHCOM:
 		ssh2key = import_ssh2(infilename, intype, passphrase, &error);
 		if (ssh2key) {
@@ -285,11 +287,9 @@ int main(int argc, char **argv)
 		continue;
 	    }
 
-	    if (ssh2key)
-	    {
-		char* fingerprint = ssh2key->alg->fingerprint(ssh2key->data);
-		if (fingerprint)
-		{
+	    if (ssh2key) {
+		char* fingerprint = ssh2_fingerprint(ssh2key->alg, ssh2key->data);
+		if (fingerprint) {
 		    fzprintf(sftpReply, "%s", fingerprint);
 		    continue;
 		}
