@@ -8,8 +8,10 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <assert.h>
+
 #include <termios.h>
 #include <unistd.h>
+#include <fcntl.h>
 
 #include "putty.h"
 #include "storage.h"
@@ -65,12 +67,16 @@ void update_specials_menu(void *frontend)
 {
 }
 
-void timer_change_notify(long next)
+void notify_remote_exit(void *frontend)
 {
 }
 
-int verify_ssh_host_key(void *frontend, char *host, int port, char *keytype,
-                        char *keystr, char *fingerprint,
+void timer_change_notify(unsigned long next)
+{
+}
+
+int verify_ssh_host_key(void *frontend, char *host, int port,
+                        const char *keytype, char *keystr, char *fingerprint,
                         void (*callback)(void *ctx, int result), void *ctx)
 {
     int ret;
@@ -237,7 +243,7 @@ int askalg(void *frontend, const char *algtype, const char *algname,
  * Ask whether to wipe a session log file before writing to it.
  * Returns 2 for wipe, 1 for append, 0 for cancel (don't log).
  */
-int askappend(void *frontend, Filename filename,
+int askappend(void *frontend, Filename *filename,
 	      void (*callback)(void *ctx, int result), void *ctx)
 {
     static const char msgtemplate[] =
@@ -258,11 +264,11 @@ int askappend(void *frontend, Filename filename,
 
 //FZ premsg(&cf);
     if (console_batch_mode) {
-	fprintf(stderr, msgtemplate_batch, FILENAME_MAX, filename.path);
+	fprintf(stderr, msgtemplate_batch, FILENAME_MAX, filename->path);
 	fflush(stderr);
 	return 0;
     }
-    fprintf(stderr, msgtemplate, FILENAME_MAX, filename.path);
+    fprintf(stderr, msgtemplate, FILENAME_MAX, filename->path);
     fflush(stderr);
 
     {
@@ -332,19 +338,48 @@ void logevent(void *frontend, const char *string)
 //FZ postmsg(&cf);
 }
 
-/*FZ 
-static void console_data_untrusted(const char *data, int len)
+/*
+ * Special functions to read and print to the console for password
+ * prompts and the like. Uses /dev/tty or stdin/stderr, in that order
+ * of preference; also sanitises escape sequences out of the text, on
+ * the basis that it might have been sent by a hostile SSH server
+ * doing malicious keyboard-interactive.
+ */
+
+static void console_open(FILE **outfp, int *infd)
+{
+    /*int fd;
+
+    if ((fd = open("/dev/tty", O_RDWR)) >= 0) {
+        *infd = fd;
+        *outfp = fdopen(*infd, "w");
+    } else*/ {
+        *infd = 0;
+        *outfp = stderr;
+    }
+}
+static void console_close(FILE *outfp, int infd)
+{
+    if (outfp != stderr)
+        fclose(outfp);             /* will automatically close infd too */
+}
+/*
+static void console_prompt_text(FILE *outfp, const char *data, int len)
 {
     int i;
+
     for (i = 0; i < len; i++)
 	if ((data[i] & 0x60) || (data[i] == '\n'))
-	    fputc(data[i], stdout);
-    fflush(stdout);
+	    fputc(data[i], outfp);
+    fflush(outfp);
 }*/
 
-int console_get_userpass_input(prompts_t *p, unsigned char *in, int inlen)
+int console_get_userpass_input(prompts_t *p, const unsigned char *in,
+                               int inlen)
 {
     size_t curr_prompt;
+    FILE *outfp = NULL;
+    int infd;
 
     /*
      * Zero all the results, in case we abort half-way through.
@@ -352,20 +387,22 @@ int console_get_userpass_input(prompts_t *p, unsigned char *in, int inlen)
     {
 	int i;
 	for (i = 0; i < p->n_prompts; i++)
-	    memset(p->prompts[i]->result, 0, p->prompts[i]->result_len);
+            prompt_set_result(p->prompts[i], "");
     }
 
-    if (console_batch_mode)
+    if (p->n_prompts && console_batch_mode)
 	return 0;
 
+    console_open(&outfp, &infd);
+
     /*
      * Preamble.
      */
     /* We only print the `name' caption if we have to... */
     if (p->name_reqd && p->name)
-	fzprintf_raw_untrusted(sftpRequestInstruction, p->name);
+	fzprintf_raw_untrusted(sftpRequestPreamble, p->name);
     else
-	fzprintf_raw_untrusted(sftpRequestInstruction, "");
+	fzprintf_raw_untrusted(sftpRequestPreamble, "");
 
     /* ...but we always print any `instruction'. */
     if (p->instruction)
@@ -376,35 +413,55 @@ int console_get_userpass_input(prompts_t *p, unsigned char *in, int inlen)
     for (curr_prompt = 0; curr_prompt < p->n_prompts; curr_prompt++) {
 
 	struct termios oldmode, newmode;
-	int i;
+	int len;
 	prompt_t *pr = p->prompts[curr_prompt];
 
-	tcgetattr(0, &oldmode);
+	tcgetattr(infd, &oldmode);
 	newmode = oldmode;
 	newmode.c_lflag |= ISIG | ICANON;
 //	if (!pr->echo)
 	    newmode.c_lflag &= ~ECHO;
 //	else
 //	    newmode.c_lflag |= ECHO;
-	tcsetattr(0, TCSANOW, &newmode);
+	tcsetattr(infd, TCSANOW, &newmode);
 
 	fzprintf_raw_untrusted(sftpRequest, "%d%s\n", (int)sftpReqPassword, pr->prompt);
 
-	i = read(0, pr->result, pr->result_len - 1);
+        len = 0;
+        while (1) {
+            int ret;
 
-	tcsetattr(0, TCSANOW, &oldmode);
+            prompt_ensure_result_size(pr, len * 5 / 4 + 512);
+            ret = read(infd, pr->result + len, pr->resultsize - len - 1);
+            if (ret <= 0) {
+                len = -1;
+                break;
+            }
+            len += ret;
+            if (pr->result[len - 1] == '\n') {
+                len--;
+                break;
+            }
+        }
 
-	pr->result[i--] = 0;
-	while (i >= 0 && (pr->result[i] == '\r' || pr->result[i] == '\n'))
-	    pr->result[i--] = '\0';
+	tcsetattr(infd, TCSANOW, &oldmode);
 
-//	if (!pr->echo)
-//	    fputs("\n", stdout);
+	//if (!pr->echo)
+	//    console_prompt_text(outfp, "\n", 1);
 
+        if (len < 0) {
+            console_close(outfp, infd);
+            return 0;                  /* failure due to read error */
+        }
+
+	pr->result[len--] = 0;
+	while (len >= 0 && (pr->result[len] == '\r' || pr->result[len] == '\n'))
+	    pr->result[len--] = '\0';
     }
 
-    return 1; /* success */
+    console_close(outfp, infd);
 
+    return 1; /* success */
 }
 
 void frontend_keypress(void *handle)
@@ -424,8 +481,6 @@ int is_interactive(void)
  * X11-forwarding-related things suitable for console.
  */
 
-const char platform_x11_best_transport[] = "unix";
-
 char *platform_get_x_display(void) {
     return dupstr(getenv("DISPLAY"));
 }
